import { TransferState, Wormhole, amount, isAttested, isRedeemed, isSourceFinalized, isSourceInitiated, routes, signSendWait, } from "@wormhole-foundation/sdk-connect";
import "@wormhole-foundation/sdk-definitions-ntt";
import { NttRoute } from "./types.js";
export function nttAutomaticRoute(config) {
    class NttRouteImpl extends NttAutomaticRoute {
        static config = config;
    }
    return NttRouteImpl;
}
export class NttAutomaticRoute extends routes.AutomaticRoute {
    NATIVE_GAS_DROPOFF_SUPPORTED = true;
    IS_AUTOMATIC = true;
    // @ts-ignore
    // Since we set the config on the static class, access it with this param
    // the NttManualRoute.config will always be empty
    staticConfig = this.constructor.config;
    static config = { tokens: {} };
    static meta = { name: "AutomaticNtt" };
    static supportedNetworks() {
        return NttRoute.resolveSupportedNetworks(this.config);
    }
    static supportedChains(network) {
        return NttRoute.resolveSupportedChains(this.config, network);
    }
    static async supportedSourceTokens(fromChain) {
        return NttRoute.resolveSourceTokens(this.config, fromChain);
    }
    static async supportedDestinationTokens(sourceToken, fromChain, toChain) {
        return NttRoute.resolveDestinationTokens(this.config, sourceToken, fromChain, toChain);
    }
    static isProtocolSupported(chain) {
        return chain.supportsProtocol("Ntt");
    }
    getDefaultOptions() {
        return NttRoute.AutomaticOptions;
    }
    async isAvailable() {
        const nttContracts = NttRoute.resolveNttContracts(this.staticConfig, this.request.source.id);
        const ntt = await this.request.fromChain.getProtocol("Ntt", {
            ntt: nttContracts,
        });
        return ntt.isRelayingAvailable(this.request.to.chain);
    }
    async validate(params) {
        const options = params.options ?? this.getDefaultOptions();
        const gasDropoff = amount.parse(options.gasDropoff ?? "0.0", this.request.toChain.config.nativeTokenDecimals);
        const amt = amount.parse(params.amount, this.request.source.decimals);
        const validatedParams = {
            amount: params.amount,
            normalizedParams: {
                amount: amt,
                sourceContracts: NttRoute.resolveNttContracts(this.staticConfig, this.request.source.id),
                destinationContracts: NttRoute.resolveNttContracts(this.staticConfig, this.request.destination.id),
                options: {
                    queue: false,
                    automatic: true,
                    gasDropoff: amount.units(gasDropoff),
                },
            },
            options,
        };
        return { valid: true, params: validatedParams };
    }
    async quote(params) {
        const { fromChain, toChain } = this.request;
        const ntt = await fromChain.getProtocol("Ntt", {
            ntt: params.normalizedParams.sourceContracts,
        });
        const deliveryPrice = await ntt.quoteDeliveryPrice(toChain.chain, params.normalizedParams.options);
        return {
            success: true,
            params,
            sourceToken: {
                token: this.request.source.id,
                amount: params.normalizedParams.amount,
            },
            destinationToken: {
                token: this.request.destination.id,
                amount: amount.parse(params.amount, this.request.destination.decimals),
            },
            relayFee: {
                token: Wormhole.tokenId(fromChain.chain, "native"),
                amount: amount.fromBaseUnits(deliveryPrice, fromChain.config.nativeTokenDecimals),
            },
            destinationNativeGas: amount.fromBaseUnits(params.normalizedParams.options.gasDropoff ?? 0n, toChain.config.nativeTokenDecimals),
        };
    }
    async initiate(signer, quote) {
        const { params } = quote;
        const { fromChain, from, to } = this.request;
        const sender = Wormhole.parseAddress(signer.chain(), signer.address());
        const ntt = await fromChain.getProtocol("Ntt", {
            ntt: params.normalizedParams.sourceContracts,
        });
        const initXfer = ntt.transfer(sender, amount.units(params.normalizedParams.amount), to, params.normalizedParams.options);
        const txids = await signSendWait(fromChain, initXfer, signer);
        return {
            from: from.chain,
            to: to.chain,
            state: TransferState.SourceInitiated,
            originTxs: txids,
            params,
        };
    }
    async *track(receipt, timeout) {
        if (isSourceInitiated(receipt) || isSourceFinalized(receipt)) {
            const { txid } = receipt.originTxs[receipt.originTxs.length - 1];
            const vaa = await this.wh.getVaa(txid, "Ntt:WormholeTransfer", timeout);
            if (!vaa)
                throw new Error("No VAA found for transaction: " + txid);
            const msgId = {
                chain: vaa.emitterChain,
                emitter: vaa.emitterAddress,
                sequence: vaa.sequence,
            };
            receipt = {
                ...receipt,
                state: TransferState.Attested,
                attestation: {
                    id: msgId,
                    attestation: vaa,
                },
            };
            yield receipt;
        }
        const { toChain } = this.request;
        const ntt = await toChain.getProtocol("Ntt", {
            ntt: receipt.params.normalizedParams.destinationContracts,
        });
        if (isAttested(receipt)) {
            const { attestation: { attestation: vaa }, } = receipt;
            if (await ntt.getIsApproved(vaa)) {
                receipt = {
                    ...receipt,
                    state: TransferState.DestinationInitiated,
                    // TODO: check for destination event transactions to get dest Txids
                };
                yield receipt;
            }
        }
        if (isRedeemed(receipt)) {
            const { attestation: { attestation: vaa }, } = receipt;
            if (await ntt.getIsExecuted(vaa)) {
                receipt = {
                    ...receipt,
                    state: TransferState.DestinationFinalized,
                };
                yield receipt;
            }
        }
        yield receipt;
    }
}
//# sourceMappingURL=automatic.js.map