import { TransferState, Wormhole, amount, isAttested, isRedeemed, isSourceFinalized, isSourceInitiated, routes, signSendWait, } from "@wormhole-foundation/sdk-connect";
import "@wormhole-foundation/sdk-definitions-ntt";
import { NttRoute } from "./types.js";
export function nttManualRoute(config) {
    class NttRouteImpl extends NttManualRoute {
        static config = config;
    }
    return NttRouteImpl;
}
export class NttManualRoute extends routes.FinalizableRoute {
    NATIVE_GAS_DROPOFF_SUPPORTED = false;
    IS_AUTOMATIC = false;
    // @ts-ignore
    // Since we set the config on the static class, access it with this param
    // the NttManualRoute.config will always be empty
    staticConfig = this.constructor.config;
    static config = { tokens: {} };
    static meta = { name: "ManualNtt" };
    static supportedNetworks() {
        return NttRoute.resolveSupportedNetworks(this.config);
    }
    static supportedChains(network) {
        return NttRoute.resolveSupportedChains(this.config, network);
    }
    static async supportedSourceTokens(fromChain) {
        return NttRoute.resolveSourceTokens(this.config, fromChain);
    }
    static async supportedDestinationTokens(sourceToken, fromChain, toChain) {
        return NttRoute.resolveDestinationTokens(this.config, sourceToken, fromChain, toChain);
    }
    static isProtocolSupported(chain) {
        return chain.supportsProtocol("Ntt");
    }
    getDefaultOptions() {
        return NttRoute.ManualOptions;
    }
    async validate(params) {
        const options = params.options ?? this.getDefaultOptions();
        const amt = amount.parse(params.amount, this.request.source.decimals);
        const gasDropoff = amount.units(amount.parse(options.gasDropoff ?? "0.0", this.request.toChain.config.nativeTokenDecimals));
        const validatedParams = {
            amount: params.amount,
            normalizedParams: {
                amount: amt,
                sourceContracts: NttRoute.resolveNttContracts(this.staticConfig, this.request.source.id),
                destinationContracts: NttRoute.resolveNttContracts(this.staticConfig, this.request.destination.id),
                options: {
                    queue: false,
                    automatic: false,
                    gasDropoff,
                },
            },
            options,
        };
        return { valid: true, params: validatedParams };
    }
    async quote(params) {
        return {
            success: true,
            params,
            sourceToken: {
                token: this.request.source.id,
                amount: params.normalizedParams.amount,
            },
            destinationToken: {
                token: this.request.destination.id,
                amount: amount.parse(params.amount, this.request.destination.decimals),
            },
        };
    }
    async initiate(signer, quote) {
        const { params } = quote;
        const { fromChain, from, to } = this.request;
        const sender = Wormhole.parseAddress(signer.chain(), signer.address());
        const ntt = await fromChain.getProtocol("Ntt", {
            ntt: params.normalizedParams.sourceContracts,
        });
        const initXfer = ntt.transfer(sender, amount.units(params.normalizedParams.amount), to, params.normalizedParams.options);
        const txids = await signSendWait(fromChain, initXfer, signer);
        return {
            from: from.chain,
            to: to.chain,
            state: TransferState.SourceInitiated,
            originTxs: txids,
            params,
        };
    }
    async complete(signer, receipt) {
        if (!isAttested(receipt)) {
            if (isRedeemed(receipt))
                return receipt;
            throw new Error("The source must be finalized in order to complete the transfer");
        }
        const { toChain } = this.request;
        const ntt = await toChain.getProtocol("Ntt", {
            ntt: receipt.params.normalizedParams.destinationContracts,
        });
        const sender = Wormhole.parseAddress(signer.chain(), signer.address());
        const completeXfer = ntt.redeem([receipt.attestation.attestation], sender);
        const txids = await signSendWait(toChain, completeXfer, signer);
        return {
            ...receipt,
            state: TransferState.DestinationInitiated,
            destinationTxs: txids,
        };
    }
    async finalize(signer, receipt) {
        if (!isRedeemed(receipt))
            throw new Error("The transfer must be redeemed in order to finalize");
        const { attestation: { attestation: vaa }, } = receipt;
        const { toChain } = this.request;
        const ntt = await toChain.getProtocol("Ntt", receipt.params.normalizedParams.destinationContracts);
        const completeTransfer = ntt.completeInboundQueuedTransfer(toChain.chain, vaa.payload.nttManagerPayload, this.request.destination.id.address);
        const finalizeTxids = await signSendWait(toChain, completeTransfer, signer);
        return {
            ...receipt,
            state: TransferState.DestinationFinalized,
            destinationTxs: [...(receipt.destinationTxs ?? []), ...finalizeTxids],
        };
    }
    async *track(receipt, timeout) {
        if (isSourceInitiated(receipt) || isSourceFinalized(receipt)) {
            const { txid } = receipt.originTxs[receipt.originTxs.length - 1];
            const vaa = await this.wh.getVaa(txid, "Ntt:WormholeTransfer", timeout);
            if (!vaa)
                throw new Error("No VAA found for transaction: " + txid);
            const msgId = {
                chain: vaa.emitterChain,
                emitter: vaa.emitterAddress,
                sequence: vaa.sequence,
            };
            receipt = {
                ...receipt,
                state: TransferState.Attested,
                attestation: {
                    id: msgId,
                    attestation: vaa,
                },
            };
            yield receipt;
        }
        const { toChain } = this.request;
        const ntt = await toChain.getProtocol("Ntt", {
            ntt: receipt.params.normalizedParams.destinationContracts,
        });
        if (isAttested(receipt)) {
            const { attestation: { attestation: vaa }, } = receipt;
            if (await ntt.getIsApproved(vaa)) {
                receipt = {
                    ...receipt,
                    state: TransferState.DestinationInitiated,
                    // TODO: check for destination event transactions to get dest Txids
                };
                yield receipt;
            }
        }
        if (isRedeemed(receipt)) {
            const { attestation: { attestation: vaa }, } = receipt;
            if (await ntt.getIsExecuted(vaa)) {
                receipt = {
                    ...receipt,
                    state: TransferState.DestinationFinalized,
                };
                yield receipt;
            }
        }
        yield receipt;
    }
}
//# sourceMappingURL=manual.js.map